//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#include "DetectorConstruction.hh"

using namespace std;

DetectorConstruction::DetectorConstruction()
:constructed(false)
{
     DWx = 1.5*ParametersSetting::DimSample[0];
     DWy = 1.5*ParametersSetting::DimSample[1];
     DWz = 1.5*ParametersSetting::DimSample[2]; // world dimensions
     DWmax = max(DWx, max(DWy,DWz) );
}

DetectorConstruction::~DetectorConstruction()
{
 delete worldPhy;
}

G4VPhysicalVolume* DetectorConstruction::Construct()
{
 if(!constructed)
 { 
  G4double fWorldLength = DWmax;
  G4GeometryManager::GetInstance()->SetWorldMaximumExtent(fWorldLength);
  DefineMaterials();
  SetupGeometry();
  constructed = true;  
 }
 return worldPhy;
}

void DetectorConstruction::DefineMaterials()
{ 
 G4String name, symbol;             //a=mass of a mole;
 G4double density;            //z=mean number of protons;  
  
 G4Element* H  = new G4Element(name="Hydrogen",symbol="H" ,  1.,  1.01*g/mole);

 G4Element* N  = new G4Element(name="Nitrogen",symbol="N" ,  7., 14.01*g/mole);

 G4Element* O  = new G4Element(name="Oxygen"  ,symbol="O" ,  8.,  16.00*g/mole);

 G4Element* C  = new G4Element(name="Carbon"  ,symbol="C" ,  6.,  12.0107*g/mole);

 G4Element* Si = new G4Element(name="Silicium", symbol="Si",14.,  28.09*g/mole);

 G4Element* K = new G4Element(name="Potassium", symbol="K",  19,  39.10*g/mole);

 G4Element* Al = new G4Element(name="Aluminum", symbol="Al", 13,  26.98*g/mole);

 G4Element* Fe = new G4Element(name="Iron", symbol="Fe",  26,  55.85*g/mole);

 G4Element* Mg = new G4Element(name="Magnesium", symbol="Mg",  12, 24.31*g/mole);

 G4Element* Ca = new G4Element(name="Calcium", symbol="Ca",  20,  40.08*g/mole);

 G4Element* Na = new G4Element(name="Sodium", symbol="Na",  11,  22.99*g/mole);

 G4Element* Zr = new G4Element(name="Zirconium", symbol= "Zr",  39,  91.224*g/mole);

 G4Element* P = new G4Element(name="Phosphore", symbol= "P",  15,  30.973*g/mole);

 G4Element* Pb = new G4Element(name="Lead",symbol= "Pb", 82.,  207.19*g/mole);

 G4int ncomponents, natoms;

 G4Material* water = new G4Material(name="Water", density=1.000*g/cm3, ncomponents=2);
 water->AddElement(H, natoms=2);
 water->AddElement(O, natoms=1);
 components.push_back(water); // material index 0 --> water

 G4Material*SiO2 = new G4Material("Silice", density= 2.65*g/cm3, ncomponents=2);
 SiO2->AddElement(Si, natoms=1);
 SiO2->AddElement(O, natoms=2); 
 components.push_back(SiO2); // material index 1 --> SiO2
   
 G4Material* Al2O3 = new G4Material("Alumine", density= 4.0*g/cm3, ncomponents=2);
 Al2O3->AddElement(Al, natoms=2);
 Al2O3->AddElement(O, natoms=3);
 components.push_back(Al2O3); // material index 2 --> Al2O3

 G4Material*CaCO3 = new G4Material("Calcium_carbonate", density= 2.711*g/cm3, ncomponents=3);
 CaCO3->AddElement(Ca, natoms=1);
 CaCO3->AddElement(C, natoms=1);
 CaCO3->AddElement(O, natoms=3);
 components.push_back(CaCO3); // material index 3 --> CaCO3

 G4Material* MgCO3 = new G4Material("Dolomite", density= 3.0*g/cm3, ncomponents=3);
 MgCO3->AddElement(Mg, natoms=1);
 MgCO3->AddElement(C, natoms=1);
 MgCO3->AddElement(O, natoms=3);
 components.push_back(MgCO3); // material index 4 --> MgCO3

 G4double fractionmass=0;
 G4Material* Limestone= new G4Material("Limestone", 2.7*g/cm3, ncomponents=2);
 Limestone->AddMaterial(CaCO3, fractionmass= 0.7);
 Limestone->AddMaterial(MgCO3, fractionmass= 0.3);
 components.push_back(Limestone); // material index 5 --> Limestone

 G4Material*ZrSiO4 = new G4Material("Zircon", density= 4.*g/cm3, ncomponents=3);
 ZrSiO4->AddElement(Zr, natoms=1);
 ZrSiO4->AddElement(Si, natoms=1);
 ZrSiO4->AddElement(O, natoms=4);
 components.push_back(ZrSiO4); // material index 6 --> ZrSiO4        
  
 G4Material* KF = new G4Material("K-Feldspar", density= 2.60*g/cm3, ncomponents=4);
 KF->AddElement(Al,fractionmass = 0.0969);
 KF->AddElement(O, fractionmass = 0.4599);
 KF->AddElement(Si, fractionmass = 0.3027);
 KF->AddElement(K, fractionmass = 0.1405);
 components.push_back(KF); // material index 7 --> KF
   
 G4Material* H_apatite = new G4Material("Hydroxyapatite", density = 3.8*g/cm3, ncomponents=4);
 H_apatite->AddElement(O, natoms = 13);
 H_apatite->AddElement(Ca, natoms = 5);
 H_apatite->AddElement(P, natoms = 3);
 H_apatite->AddElement(H, natoms = 1);
 components.push_back(H_apatite); // material index 8 --> H_apatite

 G4Material* Fe2O3 = new G4Material("ferric_oxide", density= 5.242*g/cm3, ncomponents=2);
 Fe2O3->AddElement(Fe, natoms=2);
 Fe2O3->AddElement(O, natoms=3);
 components.push_back(Fe2O3); // material index 9 --> Fe2O3
  
 G4Material* FeO = new G4Material("ferrous_oxide", density= 5.745*g/cm3, ncomponents=2);
 FeO->AddElement(Fe, natoms=1);
 FeO->AddElement(O, natoms=1);
 components.push_back(FeO); // material index 10 --> FeO
  
 G4Material* MgO = new G4Material("magnesium_oxide", density= 3.58*g/cm3, ncomponents=2);
 MgO->AddElement(Mg, natoms=1);
 MgO->AddElement(O, natoms=1);
 components.push_back(MgO); // material index 11 --> MgO
  
 G4Material* CaO = new G4Material("calcium_oxide", density= 3.35*g/cm3, ncomponents=2);
 CaO->AddElement(Ca, natoms=1);
 CaO->AddElement(O, natoms=1);
 components.push_back(CaO); // material index 12 --> CaO
  
 G4Material* Na2O = new G4Material("sodium_oxide", density= 2.27*g/cm3, ncomponents=2);
 Na2O->AddElement(Na, natoms=2);
 Na2O->AddElement(O, natoms=1);
 components.push_back(Na2O); // material index 13 --> Na2O
  
 G4Material* K2O = new G4Material("potassium_oxide", density= 2.35*g/cm3, ncomponents=2);
 K2O->AddElement(K, natoms=2);
 K2O->AddElement(O, natoms=1);
 components.push_back(K2O); // material index 14 --> K2O

 density = 1.290*mg/cm3; 
 G4Material* air = new G4Material(name="Air", density, ncomponents=2);
 air->AddElement(N, fractionmass=0.78);
 air->AddElement(O, fractionmass=0.22);
 components.push_back(air); // material index 15 --> air

 G4Material* vacuum = new G4Material("Vacuum", 1, 1.00794*g/mole,1.0E-25*g/cm3, kStateGas, 0.1*kelvin, 1.0E-19*pascal);
 components.push_back(vacuum); // material index 16 --> vacuum

 G4Material* lead= new G4Material(name="Lead", 11.35*g/cm3, ncomponents=1);
 lead->AddElement(Pb, natoms=1);
 components.push_back(lead); // material index 17 -> lead

  ///////////********************* new components creation ***********************///////////

 CompAt_index =0;
 for(G4int i=0;i<ParametersSetting::NbCompo;i++)
 {
  G4Material* compo; compo =  CreateNewComponent(i);
  components.push_back(compo);
 }

   ///////////********************* materials creation ***********************///////////

 MatComp_index=0;
 for(G4int i=0;i<ParametersSetting::NbMat;i++)CreateNewMaterial(i);
   
}

void DetectorConstruction::SetupGeometry()
{ 
 G4Material* vacuum = G4Material::GetMaterial("Vacuum");

 //     
 // World
 //    
    
 G4Box* worldSol = new G4Box("World",DWx/2.,DWy/2.,DWz/2.);
 G4LogicalVolume* worldLog = new G4LogicalVolume(worldSol,vacuum,"World");
 worldPhy = new G4PVPlacement(0,G4ThreeVector(),worldLog,"World",0,true,0);		
									
 G4VisAttributes* worldVisAtt= new G4VisAttributes(G4Colour(1.0,1.0,1.0)); //White
 worldVisAtt->SetVisibility(true);
 worldLog->SetVisAttributes(worldVisAtt);
 
 //                               
 // Sample
 //  
 
 G4Box* SampleSol = new G4Box("Sample",ParametersSetting::DimSample[0]/2.,
                                       ParametersSetting::DimSample[1]/2.,
                                       ParametersSetting::DimSample[2]/2.);

 G4LogicalVolume* SampleLog = new G4LogicalVolume(SampleSol,vacuum,"Sample");
 new G4PVPlacement(0,G4ThreeVector(),SampleLog,"Sample",worldLog,true,0);
			 
 G4VisAttributes* targetVisAtt= new G4VisAttributes(G4Colour(1.0,1.0,1.0)); //white
 targetVisAtt->SetForceSolid(false);
 targetVisAtt->SetVisibility(false);
 SampleLog->SetVisAttributes(targetVisAtt);

 ////////////************************  voxelisation *************************////////////

 //                               
 // X axis division
 //
 G4Box* SampleDivX_Sol = new G4Box("SampleDivX", ParametersSetting::DimVox[0]/2.,
                                                 ParametersSetting::DimSample[1]/2.,
                                                 ParametersSetting::DimSample[2]/2.);
 G4LogicalVolume* SampleDivX_Log = new G4LogicalVolume(SampleDivX_Sol,vacuum,"SampleDivX");
 VoxParamXY* ParamX = new VoxParamXY(0,0);
 new G4PVParameterised("SampleDivX", SampleDivX_Log,SampleLog, kXAxis,ParametersSetting::NbVox_byAxis[0], ParamX ,0);

 //                               
 // Y axis division
 //
 G4Box* SampleDivY_Sol = new G4Box("SampleDivY", ParametersSetting::DimVox[0]/2.,
						 ParametersSetting::DimVox[1]/2.,
                                                 ParametersSetting::DimSample[2]/2.);
 G4LogicalVolume* SampleDivY_Log = new G4LogicalVolume(SampleDivY_Sol,vacuum,"SampleDivY");
 VoxParamXY* ParamY = new VoxParamXY(1,0);
 new G4PVParameterised("SampleDivY", SampleDivY_Log,SampleDivX_Log, kYAxis,ParametersSetting::NbVox_byAxis[1], ParamY ,0);

 G4VisAttributes* DivVisAtt= new G4VisAttributes(G4Colour(1.0,1.0,1.0));
 DivVisAtt->SetForceSolid(false);
 DivVisAtt->SetVisibility(false);
 SampleDivX_Log->SetVisAttributes(DivVisAtt);
 SampleDivY_Log->SetVisAttributes(DivVisAtt);
 //                               
 // Z axis voxelisation
 //
 G4Box* Vox_Sol = new G4Box("Voxel", ParametersSetting::DimVox[0]/2.,
                                     ParametersSetting::DimVox[1]/2.,
                                     ParametersSetting::DimVox[2]/2.);
 G4LogicalVolume* Vox_Log = new G4LogicalVolume(Vox_Sol,vacuum,"Voxel");
 VoxParam_Opt* ParamVox = new VoxParam_Opt(0);
 new G4PVParameterised("Voxel", Vox_Log,SampleDivY_Log, kZAxis,ParametersSetting::NbVox_byAxis[2], ParamVox ,0);

G4cout<<G4endl<<"successful creation of the main geometry"<<G4endl;
}

G4Material* DetectorConstruction::CreateNewComponent(G4int Component_No)
{
 static G4NistManager* man = G4NistManager::Instance();
 man->SetVerbose(1);

 G4Material* NewCompnent = new G4Material(ParametersSetting::CompName[Component_No], 
					  ParametersSetting::CompDensity[Component_No]*g/cm3,
					  ParametersSetting::Comp_NbAtom[Component_No]);

 for(G4int i=0;i<ParametersSetting::Comp_NbAtom[Component_No];i++)
 NewCompnent->AddElement(man->FindOrBuildElement(ParametersSetting::Comp_ListAtom[CompAt_index+i]),
                         ParametersSetting::Comp_ListAtNumber[CompAt_index+i]);
 CompAt_index += ParametersSetting::Comp_NbAtom[Component_No];

 return NewCompnent;
}

void DetectorConstruction::CreateNewMaterial(G4int Material_No)
{
 G4int ncomponents;
 G4double fractionmass;

 G4String MatName="null"; G4double densi=0; G4double  WF=0; G4int Ncomp=0;

 MatName=ParametersSetting::MatName[Material_No];
 densi=ParametersSetting::MatDensity[Material_No];
 WF=ParametersSetting::MatWaterContent[Material_No];
 Ncomp=ParametersSetting::Mat_NbComp[Material_No];

 G4double  Wcont = WF/(100.+WF);
 G4double  densiW = densi/(1.-Wcont);
 G4double matrix_densityW= (densiW - ParametersSetting::Compacity[Material_No]*ParametersSetting::Graindensity[Material_No])
                           /(1.-ParametersSetting::Compacity[Material_No]);
 if(matrix_densityW<0)
 {
  G4cout<<" ERROR: grains mass is superior to material mass in material - "<<MatName<<G4endl
  <<"Please correct the material parameters in the pilot file"<<G4endl<<"Closing of DosiVox"<<G4endl;
  abort();
 }
 G4double Wcont_matrix = WF*densi/((1.-ParametersSetting::Compacity[Material_No])*matrix_densityW);

 G4int NcompW=Ncomp;if(Wcont > 0.)NcompW++;  
 new G4Material(MatName, densiW*g/cm3 , ncomponents=NcompW);

 if(ParametersSetting::Compacity[Material_No]>0. )
 {
  new G4Material(MatName+"_grain", ParametersSetting::Graindensity[Material_No]*g/cm3 , ncomponents=1);
  G4Material::GetMaterial(MatName+"_grain")->AddMaterial(components[ParametersSetting::CompGrainId[Material_No]] ,fractionmass= 1.);
  new G4Material(MatName+"_matrix", matrix_densityW*g/cm3 , ncomponents=NcompW);
 }

 if(Wcont >0.)
 {
  G4Material::GetMaterial(MatName)->AddMaterial(components[0] ,fractionmass= Wcont);
  if(ParametersSetting::Compacity[Material_No]>0.)
  {G4Material::GetMaterial(MatName+"_matrix")->AddMaterial(components[0], fractionmass= Wcont_matrix/100.);}
 }

 G4double fmassCorrection = 0;
 for(G4int j=0;j<Ncomp; j++)
 {
  G4int compId=0;G4double fmass=0;
  compId=ParametersSetting::Mat_ListComponents[MatComp_index+j];
  fmass=ParametersSetting::Mat_ListContents[MatComp_index+j];

  if(ParametersSetting::Compacity[Material_No]>0.)
  {
   G4double fmass_matrix=0;
   if(compId==ParametersSetting::CompGrainId[Material_No])
   {
    fmass_matrix = (fmass-100.*ParametersSetting::Graindensity[Material_No]/densi*ParametersSetting::Compacity[Material_No])
                   /(1.-ParametersSetting::Graindensity[Material_No]/densi*ParametersSetting::Compacity[Material_No]);
    if(fmass_matrix<=-1)
    {
     G4cout<<" Not enought mass of component "<<ParametersSetting::Mat_ListComponents[MatComp_index+j]
     <<" to create the grains in the material "<<MatName<<G4endl;
     G4cout<<"grain component mass fraction: "<<fmass<<" %"<<G4endl
     <<"remaining mass fraction of component after grains creation: "<<fmass_matrix<<" %"<<G4endl
     <<"Closing of DosiVox"<<G4endl;
     abort();
    }
   }
   else fmass_matrix = fmass/(1-ParametersSetting::Graindensity[Material_No]/densi*ParametersSetting::Compacity[Material_No]);

   if(fmass_matrix>100)
   {
    G4cout<<" ERROR: the grain component mass in the material - "<<MatName<<" - is not enought to create the grains"<<G4endl
    <<"Please correct the material parameters in the pilot file"<<G4endl<<"Closing of DosiVox"<<G4endl;
    abort();
   }

   //                               
   // mass fraction correction for a small negative value
   //
   if(fmassCorrection>0)
   {
    fmass_matrix -= fmassCorrection; G4cout<<"correction of component mass required for component "
    <<ParametersSetting::Mat_ListComponents[MatComp_index+j]<<G4endl;
    fmassCorrection =0;
   }
   if((fmass_matrix<=0)&&(fmass_matrix>-1))
   {
    fmassCorrection = 0.1-fmass_matrix; G4cout<<"calculation of component mass correction"<<G4endl;
    fmass_matrix = 0.1;
   }

   G4Material::GetMaterial(MatName+"_matrix")->AddMaterial(components[compId] ,fractionmass= fmass_matrix*(1.-Wcont_matrix/100.)/100.);
  }
  G4Material::GetMaterial(MatName)->AddMaterial(components[compId] ,fractionmass= fmass*(1-Wcont)/100.);
 }
 MatComp_index += Ncomp;    
}

