//
// ****************************************** Licence and Disclaimer ******************************************** //
//
// DosiVox is provided under the terms and conditions of the DosiVox Software License (GNU GPL3), included in
// the file DosiVox_Software_Licence and available at http://www.iramat-crp2a.cnrs.fr/spip/spip.php?//article144.
//
// Neither the authors of this software system, nor their employing institutes,nor the agencies providing
// financial support for this  work  make  any representation or  warranty, express or implied, regarding 
// this  software system or assume any liability for its use.  Please see the license in the file
// DosiVox_Software_Licence  and URL above for the full disclaimer and the limitation of liability.
//
// By using,  copying,  modifying or  distributing the software (or any work based  on the software)  you 
// agree  to acknowledge its use  in  resulting acceptance of all terms of the DosiVox Software license.
//
// DosiVox (version 1.0 )is a free software (GNU GPL3 licence, see above) based on the Geant4 C++ libraries
// (available at  http://geant4.org/ and responding to the Geant4 Software Licence available at 
// http://cern.ch/geant4/license) (version Geant4 10.1). It is developed by L.Martin, N.Mercier and S.Incerti at
// the IRAMAT-CRP2A,the CENBG and the university Bordeaux-Montaigne. DosiVox allows constructing voxelised models
// of archeological objects and simulating the radioactivity for dose calculation with a Monte-Carlo approach. 
// These dose geometry and parameters, allowing to create simulation by writing or editing a text file. DosiVox is 
// developped and tested on a Scientific Linux (Red Hat) virtual machine available at 
// http://geant4.in2p3.fr/spip.php?rubrique8 .
// ************************************************************************************************************** //
//
// Please cite the following paper if you use this software:
// Martin, L., Incerti, S., Mercier, N., 2015. 
// DosiVox : a Geant 4-based software for dosimetry simulations relevant to luminescence and ESR dating techniques. 
// Ancient TL 33, 1-10. 

#include "VoxParam_Opt.hh"
#include "G4SystemOfUnits.hh"
#include "G4PhysicalConstants.hh"


VoxParam_Opt::VoxParam_Opt(G4int world)
{
 fWorldFlag = world; // 0 for the main grid, 1 for the sub-voxelised detector

 Mat.reserve(ParametersSetting::NbMat);
 Color.reserve(ParametersSetting::NbMat); // vector initialization

 if(fWorldFlag == 0) // main voxel grid
 {
  fNVoxX = ParametersSetting::NbVox_byAxis[0];
  fNVoxY = ParametersSetting::NbVox_byAxis[1];
  fNVoxZ = ParametersSetting::NbVox_byAxis[2];
 }
 else // sub-voxelised detector
 {
  fNVoxX = ParametersSetting::Object_voxelNumberXYZ[0];
  fNVoxY = ParametersSetting::Object_voxelNumberXYZ[1];
  fNVoxZ = ParametersSetting::Object_voxelNumberXYZ[2];
 }

 for(G4int i=0;i<ParametersSetting::NbMat;i++) // setting the color of each material
 {
  G4int Xcolor = (i+1)%2;
  G4int Ycolor = (((i+1)-Xcolor)/2)%2;
  G4int Zcolor = (((((i+1)-Xcolor)/2)-Ycolor)/2)%2;
  Color[i] = new G4VisAttributes(G4Colour(Xcolor,Ycolor,Zcolor));
  Color[i]->SetForceSolid(false);
  Color[i]->SetVisibility(true);
 }

 SiO2 = G4Material::GetMaterial("Silice"); // default material

 // getting all the existing material in the simulation
 for(G4int i=0;i<ParametersSetting::NbMat;i++) Mat[i]=G4Material::GetMaterial(ParametersSetting::MatName[i]);
 
 // creating the voxel shape
 fVoxel =0;
 if(fWorldFlag == 0)
 fVoxel = new G4Box("Voxel",ParametersSetting::DimVox[0]/2.,ParametersSetting::DimVox[1]/2.,ParametersSetting::DimVox[2]/2.);

 fVoxelDet = 0;
 if(fWorldFlag == 1)
  fVoxelDet = new G4Box("Det_Voxel",ParametersSetting::DimDet[0]/2.,ParametersSetting::DimDet[1]/2.,ParametersSetting::DimDet[2]/2.);

}

VoxParam_Opt::~VoxParam_Opt()
{;}

void VoxParam_Opt::ComputeTransformation(const G4int copyNo,G4VPhysicalVolume *physVol) const
{
 G4ThreeVector Pos(0,0,0);

 if(fWorldFlag ==0) Pos = G4ThreeVector(0,0,ParametersSetting::DimVox[2]*(copyNo+0.5)-ParametersSetting::DimSample[2]/2.);
 else Pos = G4ThreeVector(0,0,ParametersSetting::DimDet[2]*(copyNo+0.5)-ParametersSetting::DimVox[2]/2.);

 physVol->SetTranslation(Pos); // voxel location parameterization in the XY division (nested parameterization)

}

G4Material* VoxParam_Opt::ComputeMaterial(G4VPhysicalVolume* PVol,const G4int copyNo,const G4VTouchable* voxelTouchable)
{
 fMat=SiO2; // default material
 if(voxelTouchable==0) return Mat[0]; // useless security

 G4int Y_No = voxelTouchable->GetCopyNumber(0); 
 G4int X_No = voxelTouchable->GetCopyNumber(1); // getting the ID of the X and Y divisions

 if(fWorldFlag == 0) // setting the material in the main voxel grid
 {
  fMat = Mat[ParametersSetting::MaterialMap[X_No + Y_No*fNVoxX + copyNo*fNVoxX*fNVoxY]];
  PVol->GetLogicalVolume()->SetVisAttributes( Color[ParametersSetting::MaterialMap[X_No + Y_No*fNVoxX + copyNo*fNVoxX*fNVoxY]]);
 } 
 else // setting the material in the sub-voxelised detector
 {
  fMat = Mat[ParametersSetting::Object_MaterialMap[X_No + Y_No*fNVoxX + copyNo*fNVoxX*fNVoxY]];
  PVol->GetLogicalVolume()->SetVisAttributes( Color[ParametersSetting::Object_MaterialMap[X_No + Y_No*fNVoxX + copyNo*fNVoxX*fNVoxY]]);
 }

 return fMat;

} // voxel material parameterization

G4VSolid* VoxParam_Opt::ComputeSolid(const G4int /*copyNo*/, G4VPhysicalVolume* )
{
   if(fWorldFlag == 0) return fVoxel;
   else return fVoxelDet;       
} // voxel shape and size parameterization

G4int VoxParam_Opt::GetNumberOfMaterials() const
{
    G4int No = ParametersSetting::NbMat;
    return No;
} // number of material in the simulation



